<?php

namespace PhpCoveralls\Tests\Component\System\Git;

use PhpCoveralls\Component\System\Git\GitCommand;
use PHPUnit\Framework\TestCase;

/**
 * @covers \PhpCoveralls\Component\System\Git\GitCommand
 * @covers \PhpCoveralls\Component\System\SystemCommand
 *
 * @author Kitamura Satoshi <with.no.parachute@gmail.com>
 */
class GitCommandTest extends TestCase
{
    // getCommandPath()

    /**
     * @test
     */
    public function shouldBeGitCommand()
    {
        $object = new GitCommand();

        $expected = 'git';

        $this->assertSame($expected, $object->getCommandPath());
    }

    // getBranches()
    //

    /**
     * @test
     */
    public function shouldExecuteGitBranchCommand()
    {
        $object = $this->createGitBranchesCommandMock('git branch');
        $object->getBranches();
    }

    /**
     * @test
     */
    public function shouldReturnBranches()
    {
        $object = new GitCommand();
        $actual = $object->getBranches();

        $this->assertInternalType('array', $actual);
        $this->assertNotEmpty($actual);
    }

    // getHeadCommit()

    /**
     * @test
     */
    public function shouldExecuteGitLogCommand()
    {
        $object = $this->createGitHeadCommitCommandMock("git log -1 --pretty=format:'%H%n%aN%n%ae%n%cN%n%ce%n%s'");
        $object->getHeadCommit();
    }

    /**
     * @test
     */
    public function shouldReturnHeadCommit()
    {
        $object = new GitCommand();
        $actual = $object->getHeadCommit();

        $this->assertInternalType('array', $actual);
        $this->assertNotEmpty($actual);
        $this->assertCount(6, $actual);
    }

    // getRemotes()

    /**
     * @test
     */
    public function shouldExecuteGitRemoteCommand()
    {
        $object = $this->createGitRemotesCommandMock('git remote -v');
        $object->getRemotes();
    }

    /**
     * @test
     */
    public function shouldReturnRemotes()
    {
        $object = new GitCommand();
        $actual = $object->getRemotes();

        $this->assertInternalType('array', $actual);
        $this->assertNotEmpty($actual);
    }

    // execute()

    /**
     * @test
     * @expectedException \RuntimeException
     */
    public function throwRuntimeExceptionIfExecutedWithoutArgs()
    {
        // `git` return 1 and cause RuntimeException
        $object = new GitCommand();
        $object->execute();
    }

    // createCommand()

    /**
     * @test
     */
    public function shouldCreateCommand()
    {
        $object = new GitCommand();
        $object->setCommandPath('ls');

        $actual = $object->execute();

        $this->assertInternalType('array', $actual);
        $this->assertNotEmpty($actual);
    }

    /**
     * @param array $params
     *
     * @return GitCommand
     */
    protected function createGitBranchesCommandMock($params)
    {
        $adapter = $this->prophesize(GitCommand::class);
        $adapter
            ->getBranches()
            ->willReturn($params)
            ->shouldBeCalled();

        return $adapter->reveal();
    }

    /**
     * @param array $params
     *
     * @return GitCommand
     */
    protected function createGitHeadCommitCommandMock($params)
    {
        $adapter = $this->prophesize(GitCommand::class);
        $adapter
            ->getHeadCommit()
            ->willReturn($params)
            ->shouldBeCalled();

        return $adapter->reveal();
    }

    /**
     * @param array $params
     *
     * @return GitCommand
     */
    protected function createGitRemotesCommandMock($params)
    {
        $adapter = $this->prophesize(GitCommand::class);
        $adapter
            ->getRemotes()
            ->willReturn($params)
            ->shouldBeCalled();

        return $adapter->reveal();
    }
}
